<?php

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

jimport('joomla.application.component.model');


/**
 * Content Component Gallery Model
 *
 * @author	Louis Landry <louis.landry@joomla.org>
 * @package		Joomla
 * @subpackage	Content
 * @since 1.5
 */
class ZoomModelGallery extends JModel
{

	/**
	 * Category id
	 *
	 * @var int
	 */
	var $_id = null;


	/**
	 * Gallery data
	 *
	 * @var object
	 */
	var $_gallery = null;

	/**
	 * SubGalleries data
	 *
	 * @var object
	 */
	var $_subgalleries = null;

	/**
	 * Constructor
	 *
	 */
	function __construct()
	{
		parent::__construct();

		global $mainframe;

		$id = JRequest::getVar('catid', 0, '', 'int');
		$this->setId((int)$id);

		// here we initialize defaults for category model
		$params = &$mainframe->getParams();
		$params->def('filter',					1);
		$params->def('filter_type',				'title');
	}



	/**
	 * Method to set the gallery id
	 *
	 * @access	public
	 * @param	int	Category ID number
	 */
	function setId($id)
	{
		// Set category ID and wipe data
		$this->_id		= $id;
		$this->_gallery		= null;
		$this->_subgalleries 	= null;
		$this->_images		= null;
		$this->_total		= NULL;
	}
	
	
	/**
	 * Method to get category data for the current category
	 *
	 * @since 1.5
	 */
	function &getGallery()
	{
		global $zoom;
		// Load the Category data
		if ($this->_loadGallery())
		{
			// Initialize some variables
			$user = &JFactory::getUser();

			// Make sure the category is published
			if ( $this->_id && !$this->_gallery->_published  && !$zoom->_isAdmin) {
				JError::raiseError(404, JText::_("Resource Not Found"));
				return false;
			}

			// check whether category access level allows access
			if ( $this->_id && !$this->_gallery->isMember() ) {
				JError::raiseError(403, JText::_("ALERTNOTAUTH"));
				return false;
			}
		}
		return $this->_gallery;
	}

	function _loadGallery() {
		if ( $this->_id ) {
			global $zoom;
			if ( !$zoom->_gallery ) {
				$zoom->setGallery($this->_id);
			}
			$this->_gallery =& $zoom->_gallery;
		} else {
			$this->_gallery =  new stdclass;
			$this->_gallery->_id = 0;
			$this->_gallery->_name = '';
			$this->_gallery->_descr = '';
			$this->_gallery->_slug = '';
			$this->_gallery->_subcat_id = null;
			$this->_gallery->_subcat_slug = '';
			$this->_gallery->_username = '';
		}
		return true;
	}

	function &getSubgalleries()
	{
		global $zoom;
		if (!$this->_subgalleries && $this->_loadGallery()) {
			if ( $this->_id ) {
				$this->_subgalleries = $this->_gallery->_subcats;
			} else {
				$db =& JFactory::getDBO();
				$orderMethod = $zoom->getCatOrderMethod();
				if ($zoom->_isAdmin) {
					$db->setQuery("SELECT catid FROM #__zoom WHERE subcat_id=0 AND pos=0 ORDER BY ".$orderMethod);
				} else {
					$db->setQuery("SELECT catid FROM #__zoom WHERE subcat_id=0 AND pos=0 AND published=1 ORDER BY ".$orderMethod);
				}
				$rows = $db->loadObjectList();
				$this->_subgalleries = array();
				foreach ($rows as $row) {
					$this->_subgalleries[] = new gallery($row->catid, true);
				}
			}
		}
		return $this->_subgalleries;
	}

	function &getImages()
	{
		global $zoom;
		if ( !$this->_images && $this->_loadGallery() ) {
			$this->_total=0;
			$this->_images = array();
			if ($this->_id) {
				// Only return the images that the current user can access to
				foreach ($this->_gallery->_images as $imgid => $img) {
					if ( $img->isMember() ) {
						$this->_images[$imgid] =& $this->_gallery->_images[$imgid];
						$this->_total++;
					}
				}
			}
		}
		return $this->_images;
	}

	function getTotal()
	{
		if ($this->_total === NULL)
		{
			if ($this->_images === NULL) {
				$this->getImages();
			}
		}
		return $this->_total;
	}

	function store()
	{
		global $zoom;
		$db 	=& JFactory::getDBO();

		$catid = JRequest::getInt('catid');
		$db->setQuery("select * from #__zoom WHERE catid = $catid");
		$cat = $db->loadObject();
		if ( $zoom->_isAdmin || ( ($cat->uid == $zoom->currUID || $cat->shared == 1) &&
			(($catid && $zoom->privileges->hasPrivilege('priv_editgal')) || 
			(!$catid && $zoom->privileges->hasPrivilege('priv_creategal'))) )) {
			$zoom->setGallery($catid);
			$gallery =& $zoom->_gallery;

			// get the variables to save...
			$parent		= JRequest::getInt('parent');
			$newname	= JRequest::getVar('catname');
			$catpass	= JRequest::getVar('password');
			$keywords	= JRequest::getVar('keywords');
			$newdescr	= JRequest::getVar('catdescr','','POST','STRING',JREQUEST_ALLOWHTML);
			$hidemsg	= JRequest::getInt('hide_msg');
			$shared		= JRequest::getInt('shared');
			$published	= JRequest::getInt('published');
			$selections	= JRequest::getVar('members_sel','','POST','STRING');
			$dir		= JFolder::makeSafe(JRequest::getVar('catdir'));

			if ( isset($newname) ) {
				//Save changes
				$catdir = $gallery->_dir;

				if ($parent == 0) {
					$pos = 0;
				} else {
					$db->setQuery("SELECT pos FROM #__zoom WHERE catid=$parent");
					$row = $db->loadObject();
					$pos = $row->pos;
					$pos++;
				}

				if (!$dir) {
					$dir = $zoom->newdir();
				}
				if (!$hidemsg) {
					$hidemsg = 0;
				}
				if ($catpass != "" || !empty($catpass)) {
					$catpass = md5($catpass);
				} else {
					$catpass = "";
				}

				if (empty($selections)) {
					$selections = 1;
				} else {
					$selections = implode(',', $selections);
				}

				if ($dir != $catdir) {
					$olddir = JPATH_SITE.DS.$zoom->_CONFIG['imagepath'].$catdir;
					$newdir = JPATH_SITE.DS.$zoom->_CONFIG['imagepath'].$dir;
					if ($zoom->platform->is_dir($olddir)) {
						if ($zoom->platform->is_dir($olddir)) {
							$basedir = dirname($newdir);
							if (!$zoom->platform->is_dir($basedir)) {
								if (!$zoom->createdir($zoom->_CONFIG['imagepath'].$dir, 0777)){
								// TODO: Dir not created
								}
							}
							if ($zoom->platform->is_dir($basedir)) {
								$result = $zoom->platform->rename($olddir,$newdir);
								if ($result === TRUE) {
									$catdir = $dir;
								} else {
									$this->setError($result);
									return false;
								}
							}
						}
					}else {
						$catdir = $dir;
					}
				}

				if ( !$zoom->platform->is_dir(JPATH_SITE.DS.$zoom->_CONFIG['imagepath'].$catdir) ) {
					$html_file = "<html><body bgcolor=\"#FFFFFF\"></body></html>";
					if ( $zoom->createdir($zoom->_CONFIG['imagepath'].$catdir, 0777) ){
						$zoom->createdir($zoom->_CONFIG['imagepath'].$catdir.DS."thumbs", 0777);
						$zoom->createdir($zoom->_CONFIG['imagepath'].$catdir.DS."viewsize", 0777);
						$zoom->writefile(JPATH_SITE.DS.$zoom->_CONFIG['imagepath'].$catdir.DS."index.html", $html_file);
						$zoom->writefile(JPATH_SITE.DS.$zoom->_CONFIG['imagepath'].$catdir.DS."thumbs".DS."index.html", $html_file);
						$zoom->writefile(JPATH_SITE.DS.$zoom->_CONFIG['imagepath'].$catdir.DS."viewsize".DS."index.html", $html_file);
					}
				}

				// replace space-character with 'air'...or nothing!
				$keywords = trim(ereg_replace(" ", "", $keywords));
				$db->setQuery("UPDATE #__zoom "
				. "SET catname='".$zoom->escapeString($newname)."', catdescr='".$zoom->escapeString($newdescr)."', "
				. " catpassword='".$zoom->escapeString($catpass)."', catkeywords='".$zoom->escapeString($keywords)."', "
				. " subcat_id='$parent', pos='$pos', hideMsg='$hidemsg', shared = '$shared', published='$published', "
				. " catmembers='".$zoom->escapeString($selections)."',catdir='".$zoom->escapeString($catdir)."' "
				. "WHERE catid=".$zoom->escapeString($catid));
				$db->query();
				//Unpublish/ publish the images of a gallery too...
				//Check if there are ANY images in the gallery...
				$db->setQuery("SELECT imgid FROM #__zoomfiles WHERE catid=$catid");
				$result = $db->query();
			} else {
				$this->setError(JText::_('NONAME'));
				return false;
			}
			return true;
		} else {
			$this->setError(JText::_('ALERT_NOPERMISSIONS'));
			return false;
		}
	}

	function saveOrder($itemsType,$ordering) {
		if (!$ordering) {
			return false;
		}

		$db = &JFactory::getDBO();
		$order = explode(',',$ordering);
		switch ($itemsType) {
			case 'galleries':
				$items = $this->getSubgalleries();
				$table = '#__zoom';
				$idfield = 'catid';
				$pfield = 'subcat_id';
			break;
			case 'images':
				$items = $this->getImages();
				$table = '#__zoomfiles';
				$idfield = 'imgid';
				$pfield = 'catid';
			break;
		}

		$i=0;
		$keys = array_keys($items);
		foreach ( $items as $item ) {
			$item->_custom_order = $order[$i];
			$iorder = intval($order[$i]);
			$query = "UPDATE `$table` SET custom_order=$i WHERE $idfield={$keys[$iorder]} AND $pfield={$this->_id}";
			$i++;
			$db->setQuery($query);
			# echo $db->getQuery(). "\n";
			$db->query();
		}
	}
}
